/* Copyright (c) 1991 by John Atwood deVries II. */
/* For copying and distribution information, see the file COPYING. */

#include "../h/version.h"
#include "server.h"
#include "externs.h"
#include <string.h>

/* #define MAX_IDLE 120 */

/* function declarations */
extern void mdb(char *);
extern void sendexit(int);
extern void sends_cmdout(int, char *);
extern void disconnectuser(int);
extern void clear_user_item(int);
extern int find_group(char *);
extern void s_status_group(int, int, int, char *, char *);
extern void clear_group_item(int);
extern int check_mods(int);
extern void dispatch(int, char *);


void
c_packet(pkt)
	char   *pkt;
{
	/* the server doesn't handle client packets */
	mdb("server erroneously got client packet");
}


void
c_didpoll()
{
	/* do interesting nothing for now */
	mdb("server erroneously got client didpoll call");
}


void
c_lostcon()
{
	mdb("server erroneously got client lost connection packet");
}

void
s_didpoll(n)
	int     n;
{
	int     i;
	time_t  TheTime;

	if (n == 1) {
		/* do nothing on the frequent polls */
	} else {
		TheTime = time(NULL);
		/* BUG!  kill really ought to be a stack. if several fn -w's
		 * come in all at once, then we ought to be able to stack them */

		if (S_kill != 0) {
			sends_cmdout(n, ".... That ought to do it!");
			sendexit(S_kill);
			mdb("[FYI] killed someone");
			u_tab[S_kill].login = 0;
			S_kill = 0;
		}
#if MAX_IDLE > 0
/*		mdb("[POLL] checking timeouts"); */
		/* loop through real users */
		for (i = 0; i < MAX_REAL_USERS; i++) {
			/* look at folx who are logged in */
			if (u_tab[i].login == 1) {
				if ((TheTime - u_tab[i].t_recv) > MAX_IDLE) {
					/* kill that puppy */
					sprintf(mbuf,
					    "[TIMEOUT] %d", i);
					mdb(mbuf);
					u_tab[i].flags |= USER_IDLED_OUT;
					disconnectuser(i);
				}
			}
		}
#endif
	}
}

void
c_userchar()
{
	mdb("[Error] server erroneously got client userchar call");
}

void
s_lost_user(n)
	int     n;		/* fd of that user */
{
	int     num_left;
	int     was_mod;
	int     ogi;
	char    t_grp[MAX_GROUPLEN + 1];
	char    t_name[MAX_NICKLEN + 1];
	int     how_many;
	int     len;
	int	u_flags;

	sprintf(mbuf, "[LOST] %d", n);
	mdb(mbuf);
	/* Make sure the user was logged in at all */
	if (u_tab[n].login == 0) {
		/* they were NOT logged in */
		/* just clear their entry and return */
		clear_user_item(n);
		return;
	}
	/* save stuff we want to know about the user */
	len = strlen(u_tab[n].group);
	how_many = (MAX_GROUPLEN > len) ? len : MAX_GROUPLEN;
	bzero(t_grp, MAX_GROUPLEN + 1);
	strncpy(t_grp, u_tab[n].group, how_many);
	u_flags = u_tab[n].flags;

	len = strlen(u_tab[n].nickname);
	how_many = (MAX_NICKLEN > len) ? len : MAX_NICKLEN;

	bzero(t_name, MAX_NICKLEN + 1);
	strncpy(t_name, u_tab[n].nickname, how_many);


	/* At least they were logged in */

	/* find out what group they are in */
	/* note: they may be moderator of a group they are NOT in */

	ogi = find_group(t_grp);

	/* decrease the number of users in that group */
	g_tab[ogi].num_users--;

	/* check if there are any users left in present group */

	if (g_tab[ogi].num_users > 0) {
		/* tell remaining group members user left */
		if (u_flags & USER_IDLED_OUT)
			sprintf(mbuf, "%s (%s@%s) has idled out.",
				t_name, u_tab[n].loginid, u_tab[n].nodeid);
		else
			sprintf(mbuf, "%s (%s@%s) just signed off.",
				t_name, u_tab[n].loginid, u_tab[n].nodeid);
		if (g_tab[ogi].volume != QUIET) {
			s_status_group(2, 0, ogi, "Signoff", mbuf);
		}
	} else {
		/* otherwise zap the group he was in */
		clear_group_item(ogi);
	}

	/* clear the user from the user table to keep searches from finding
	 * them. */
	clear_user_item(n);

	/* check if user was mod of any group */
	was_mod = check_mods(n);

	/* if was mod, check if there are any users left in moderated group */
	if (was_mod >= 0) {
		num_left = g_tab[was_mod].num_users - 1;
		if (num_left > 0) {
			/* if, additionally, the user was mod, tell them that
			 * too. */
			if (g_tab[was_mod].volume != QUIET) {
				s_status_group(2, 0, was_mod, "Signoff",
				    "Your group moderator signed off.");
			}
		}
		g_tab[was_mod].mod = -1;	/* no mod now */
	}
}

void
s_packet(x, pkt)
	int     x;
	char   *pkt;
{
	/* go figure out what kind of packet */
	dispatch(x, ++pkt);
}
