/* Copyright (c) 1991 by John Atwood deVries II. */
/* For copying and distribution information, see the file COPYING. */

#include "server.h"
#include "externs.h"
#include "groups.h"
#include "../h/protocol.h"
#include "../h/version.h"
#include <sys/wait.h>
#include <sys/file.h>
#include <sys/dir.h>
#include <strings.h>
#include <stdlib.h>
#include <unistd.h>

/* function prototypes */
extern void init_groups();
extern void clear_users();
extern int readnicks();
extern void fill_user_entry(int, char *, char *, char *, char *, char *,
		int, int);
extern void fill_group_entry(int, char *, char *, int, int, int, int);
extern int makenewport(char *, int);
extern void setpolldelay(int);
extern void setpolltimeout(int);
extern void trapsignals();
extern int readtable();
extern int serverserve();
extern void mdb(char *);
extern void disconnectuser(int);
void    create_server();

int	portnumber = DEFAULTPORT;

int
main(argc, argv)
	int argc;
	char **argv;
{
	extern int optind;
	extern int opterr;
	extern char *optarg;
	int     ret;

	if ((program = strrchr(argv[0], '/')) != NULL)
		program++;
	else
		program = argv[0];
	opterr = 0;
	while ((ret = getopt(argc, argv, "dp:")) != -1) {
		switch (ret) {
		case 'd':
			fprintf(stderr, "%s: debugging enabled\n", program);
			debug++;
			break;
		case 'p':
			if (sscanf(optarg, "%d", &portnumber) != 1) {
				fprintf(stderr,
					"%s: -p requires a port number\n",
					program);
				portnumber = DEFAULTPORT;
			}
			break;
		default:
			fprintf(stderr, "usage: %s [-d] [-p port]\n", program);
			exit(1);
		}
	}

	setbuf(stdout, (char *) 0);

	if (debug == 0) {
		setuid(ICB_UID);
		setgid(ICB_GID);
		if (fork())
			exit(0);
	}


	/* get our hostname */
	if ((ret = gethostname(thishost, sizeof(thishost))) < 0) {
		perror("Main: hostname:");
		exit(1);
	}
	/* initialize out tables  */
	init_groups();
	clear_users();
	numnicks = readtable();
	dumptable(numnicks);

	/* invent our resident automagical user */
	create_server();

	/***********************************************
	Daemons should not write to stdout. tjk

	sprintf(mbuf, "ICB revision %s. \r\n", VERSION);
	write(1, mbuf, strlen(mbuf));
	sprintf(mbuf, "There are %d users possible.\n", MAX_USERS);
	write(1, mbuf, strlen(mbuf));
	sprintf(mbuf, "Of those, %d are real.\n", MAX_REAL_USERS);
	write(1, mbuf, strlen(mbuf));

	*********************************************************/

	/* stuff to start server here */
	/* first, create the advertised port */

	if (makenewport(DEFAULTHOST, portnumber) < 0) {
		perror("makenewport failed");
		exit(1);
	}
	/* then set up various initial delays */

	setpolldelay(POLL_DELAY);
	setpolltimeout(IDLE_POLLWAIT);

	/* trap signals we can handle */
	trapsignals();

	mdb("about to start serve loop");
	/* start the serve loop */
	if (serverserve() < 0) {
		mdb("server serve failed");
		perror("serverserve failed");
		exit(1);
	}
	mdb("after serverserve loop");

	/* all's well that ends */
	exit(0);
}

void
icbexit()
{
	int     user;

	mdb("someone told us to exit");
	for (user = 0; user < MAX_REAL_USERS; user++) {
		if (u_tab[user].login == 1) {
			disconnectuser(user);
		}
	}
	exit(0);
}

void
create_server()
{
	time_t  TheTime;

	TheTime = time(NULL);
	fill_user_entry(NICKSERV, "icb_server", DEFAULTHOST,
	    "Server", "", "control", TheTime, 0);
	u_tab[NICKSERV].login = 1;
	u_tab[NICKSERV].t_on = TheTime;
	u_tab[NICKSERV].t_recv = TheTime;
	strcpy(u_tab[NICKSERV].realname, "registered");
	fill_group_entry(0, "control", "...here to serve you!", INVISIBLE,
	    RESTRICTED, NICKSERV, QUIET);
	mdb("fake server created and added.");
}
