/* Copyright (c) 1991 by John Atwood deVries II. */

/* handle various messages from the client */

#include "server.h"
#include "externs.h"
#include "../h/version.h"
#include "lookup.h"
#include "groups.h"
#include <strings.h>

/* function prototypes */
extern void mdb(char *);
extern char *getremotename(int);
extern int split(char *);
extern int find_group(char *);
extern void senderror(int, char *);
extern void s_send_group(int);
extern void s_who(int, int);
extern int find_user(char *);
extern void filternickname(char *);
extern void fill_user_entry(int, char *, char *, char *, char *, char *,
		int, int);
extern int check_auth(int);
extern int s_change(int, int);
extern int nicklookup(int, char *);
extern void send_loginok(int);
extern void s_motd(int, int);
extern int lookup(char *, char **);
extern void s_drop(int, int);
extern void s_shutdown(int, int);
extern void s_wall(int, int);
extern void s_beep(int, int);
extern void s_cancel(int, int);
extern void s_invite(int, int);
extern void s_pass(int, int);
extern void s_boot(int, int);
extern void s_status(int, int);
extern void s_topic(int, int);
extern void s_personal(int, int);
extern void s_echoback(int, int);
extern void s_name(int, int);
extern void s_version(int, int);
extern void s_info(int, int);
extern void putl(char *, int);



/* open message */
/*
   note that we need to know who sent the message.  it doesn't
   come with the sender's nick, so we have to reformat the message
   before sending it to the group
*/

void
openmsg(n, pkt)
	int     n;
	char   *pkt;
{
	time_t  TheTime;
	int     gi;

	if (u_tab[n].login == 1) {
		sprintf(mbuf, "[OPEN] %d", n);
		mdb(mbuf);
		/* record the time */
		TheTime = time(NULL);
		u_tab[n].t_recv = TheTime;

		if (split(pkt) != 1) {
			mdb("[Error] got bad open message packet");
		} else {
			gi = find_group(u_tab[n].group);
			if (g_tab[gi].volume == QUIET) {
				senderror(n,
				    "Open messages not permitted in quiet groups.");
			} else {
				s_send_group(n);
			}
		}
	} else {
		mdb("[Error] cannot send open messages until logged in");
	}
}


/*
   they sent us a login message. we need to put their info into
   the user information table and then send them back a loginok message.
   for any field they didn't specify, come up with a plausible default.
   we also need to handle "w" vs. "login"
   we also need to dump them if they try using a:
	nickname already in use -or-
	group they aren't allowed to enter.
   their loginid may NOT be blank or null
*/
int
loginmsg(n, pkt)
	int     n;		/* socket on which they sent message */
	char   *pkt;		/* packet buffer */
{
	int     num_fields;
	char    which_group[MAX_GROUPLEN + 3];
	char    temp[MAX_GROUPLEN + 3];
	int     len;
	int     ret;
	int     how_many;
	time_t  TheTime;
	int     initialVisibility = VISIBLE;
	int     gi;
	char   *cp;

	sprintf(mbuf, "[LOGIN] %d", n);
	mdb(mbuf);
	mdb(pkt);
	cp = getremotename(n);
	mdb(cp);
	u_tab[n].login = 0;
/*	mdb("login is zero"); */
	/* just in case */
	TheTime = time(NULL);
/*	mdb("got the time"); */

	num_fields = split(pkt);

	len = strlen(fields[2]);
/*	mdb("len set"); */

	bzero(temp, MAX_GROUPLEN + 3);
	bzero(which_group, MAX_GROUPLEN + 3);

/*	mdb("temp and which_group zapped"); */

	if (len == 0) {		/* no group specified, give them one */
		strcpy(which_group, "1");
	} else {
		how_many = (len > (MAX_GROUPLEN + 2)) ? (MAX_GROUPLEN + 2) : len;
		strncpy(temp, fields[2], how_many);
		/* (len = 1) --> visible and unchanged (len = 2) && temp[0] ==
		 * "."   --> secret and 2nd char (len = 2) && temp[0] != "."
		 * --> visible and unchanged (len >= 3) && temp[0] == "."
		 * temp[1] == "." --> invisible and 3rd char on temp[1] != "."
		 * --> secret and 2nd char on (len >= 3) && temp[0] != "." -->
		 * visible and unchanged */
		len = strlen(temp);
		switch (len) {
		case 1:
			/* visible and unchanged */
			strcpy(which_group, temp);
			break;
		case 2:
			if (temp[0] == '.') {
				initialVisibility = SECRET;
				strcpy(which_group, &(temp[1]));
			} else {
				strcpy(which_group, temp);
			}
			break;
		default:
			if (temp[0] == '.') {
				if (temp[1] == '.') {
					initialVisibility =
					    INVISIBLE;
					strcpy(which_group,
					    &(temp[2]));
				} else {
					initialVisibility =
					    SECRET;
					strcpy(which_group,
					    &(temp[1]));
				}
			} else {
				/* visible and unchanged */
				len = strlen(temp);
				how_many = (len > MAX_GROUPLEN) ? MAX_GROUPLEN : len;
				strncpy(which_group, temp, how_many);
			}
			break;
		}

	}
/*
 mdb("which_group");
 mdb(which_group);
*/

	if (strcasecmp(fields[3], "w") == 0) {
		/* who only */
/* mdb("who only"); */
		fields[0][0] = 'w';
		fields[0][1] = '\000';
		fields[1][0] = '\000';
		s_who(n, 2);
		return -1;	/* fail */
	} else {
		/* regular login */

		/* make sure the nickname hasn't already been taken */
/* mdb("checking if name taken"); */
		if (find_user(fields[1]) >= 0) {
			/* oops, someone already has this nick */
			senderror(n, "Nickname already in use.");
			return (-1);
		}
/* mdb("checking if name null"); */

		/* make sure it isn't a null nickname */
		if (strlen(fields[1]) <= 0) {
			/* oops, too short */
			senderror(n, "Nickname may not be null.");
			return -1;
		}
		/* make sure it isn't a blank loginid if (findspace(fields[0])
		 * == '\0') { senderror(n,"Loginid must not be blank.");
		 * return -1; } */

/* get rid of nasty characters in the nickname */
/* mdb("filtering nickname"); */
		filternickname(fields[1]);


		/* fill in what we can */
/* mdb("filling in user entry"); */
		fill_user_entry(n, fields[0], cp, fields[1],
		    fields[4], "", 0, 0);

		sprintf(mbuf, "%s@%s", fields[0], cp);
		mdb(mbuf);

		/* make sure they are allowed to use this nickname */
		if (strcasecmp(fields[1], "admin") == 0) {
			if (!check_auth(n)) {
				/* oops, password is wrong */
				senderror(n, "Nickname already in use.");
				return (-1);
			}
		}
		/* fake a s_change */
		strcpy(fields[1], which_group);
		if (s_change(n, 2) < 0) {
			/* login fails because can't get into that group */
			return (-1);
		}
		/* could find group slot, all is ok */
		/* fill in last members of table entry */

		/* s_change() fills in u_tab[n].group */
		/* strcpy(u_tab[n].group, which_group); */
		strcpy(which_group, u_tab[n].group);
		u_tab[n].login = 1;
		u_tab[n].t_on = TheTime;
		u_tab[n].t_recv = TheTime;
		u_tab[n].t_sent = TheTime;


		gi = find_group(which_group);

		if (strcasecmp("1", which_group) != 0) {
			/* change when not group 1 */
			/* change when no other group members */
			if (g_tab[gi].num_users < 2)
				g_tab[gi].visibility = initialVisibility;
		} else {
			g_tab[gi].visibility = VISIBLE;
		}

		/* check to see if we know this person */
/*	mdb("About to do nicklookup"); */
		ret = nicklookup(-1, u_tab[n].nickname);
		if (ret == 0) {
			/* we know this person */
			strcpy(u_tab[n].realname, "registered");
		}
		/* send loginok message */

		send_loginok(n);

		s_motd(n, 2);

		return 0;
	}
}
/* command message */

void
cmdmsg(n, pkt)
	int     n;
	char   *pkt;
{
	int     argc;
	time_t  TheTime;

	if (u_tab[n].login == 1) {

		/* record the time */
		TheTime = time(NULL);
		u_tab[n].t_recv = TheTime;

		argc = split(pkt);

		sprintf(mbuf, "[COMMAND] %d %s", n, fields[0]);
		mdb(mbuf);

		switch (lookup(fields[0], command_table)) {

		case CMD_DROP:
			s_drop(n, argc);
			break;

		case CMD_SHUTDOWN:
			s_shutdown(n, argc);
			break;

		case CMD_WALL:
			s_wall(n, argc);
			break;

		case CMD_BEEP:
			s_beep(n, argc);
			break;

		case CMD_CANCEL:
			s_cancel(n, argc);
			break;

		case CMD_G:
			s_change(n, argc);
			break;

		case CMD_INVITE:
			s_invite(n, argc);
			break;

		case CMD_PASS:
			s_pass(n, argc);
			break;

		case CMD_BOOT:
			s_boot(n, argc);
			break;

		case CMD_STATUS:
			s_status(n, argc);
			break;

		case CMD_TOPIC:
			s_topic(n, argc);
			break;

		case CMD_MOTD:
			s_motd(n, argc);
			break;

		case CMD_M:
			s_personal(n, argc);
			break;

		case CMD_ECHOBACK:
			s_echoback(n, argc);
			break;

		case CMD_NAME:
			s_name(n, argc);
			break;

		case CMD_V:
			s_version(n, argc);
			break;

		case CMD_W:
			s_who(n, argc);
			break;

		case CMD_INFO:
			s_info(n, argc);
			break;

		default:
			sprintf(mbuf,
			    "Invalid command type \"%s\"",
			    fields[0]);
			mdb(mbuf);
			senderror(n,mbuf);
		}
	} else {
		mdb("[Error] cannot issue command until logged in");
	}
}


/* pong message */

void
pong(n, pkt)
	int     n;
	char   *pkt;
{
	if (u_tab[n].login == 1) {
		mdb("[FYI] pong message");
		/* record the time */
		/* TheTime = time(NULL); u_tab[n].t_recv= TheTime; */

		S_kill = n;
	} else {
		mdb("[Error] cannot send pong message until signed on");
	}
}
