/* Copyright (c) 1990 by Carrick Sean Casey. */
/* Modifications Copyright (c) 1991 by John Atwood deVries II */
/* For copying and distribution information, see the file COPYING. */

#include <string.h>
#include <stdlib.h>
#include "server.h"

/* function prototypes */
extern STRLIST *strgetnode(char *, STRLIST *, int);
extern void mdb(char *);
extern void strunlink(STRLIST *, STRLIST **, STRLIST **);
extern void strlinkhead(STRLIST *, STRLIST **, STRLIST **);

/* name list routines */

/* add a name to the list */

/*
if you want to add a name and a value
nlput(nl, name,n)
NAMLIST * nl;
char *name;
int n;

*/

void
nlput(nl, name)
	NAMLIST *nl;
	char   *name;
{
	STRLIST *sp;

/* mdb("entered nlput"); */
/* nldump(*nl); */
	/* hunt for user within list */
	for (sp = nl->head; sp; sp = sp->next)
		if (!strcasecmp(name, sp->str)) {
			/* found user -- put at head of list */
			strunlink(sp, &nl->head, &nl->tail);
			strlinkhead(sp, &nl->head, &nl->tail);
/*			sp->val = n; */
			nl->p = nl->head;
			return;
		}
	/* user wasn't found */
/* mdb("user wasn't found"); */
	if (nl->num < nl->max) {
		/* make a new entry for the user */
/* mdb("make a new entry for them"); */
		if ((sp = (STRLIST *) malloc(sizeof(STRLIST) + strlen(name))) == NULL) {
			perror("nlput: out of memory for names");
			return;
		}
/* mdb("copying the name"); */
		strcpy(sp->str, name);
/*		sp->val = n; */
		strlinkhead(sp, &nl->head, &nl->tail);
		nl->num++;
	} else {
		/* name list full, link user to head, remove tail */
/* mdb("name list supposedly full"); */
		sp = nl->tail;
		strunlink(sp, &nl->head, &nl->tail);
		free(sp);
		nl->num--;
		if ((sp = (STRLIST *)
			malloc(sizeof(STRLIST) + strlen(name))) == NULL) {
			perror("nlput: out of memory for names");
			return;
		}
/* mdb("copying the name"); */
		strcpy(sp->str, name);
/*		sp->val = n; */
		strlinkhead(sp, &nl->head, &nl->tail);
		nl->num++;
	}
/* mdb("setting p to the head of the list"); */
	nl->p = nl->head;
}
/* return a name */
/* repeatedly called, will cycle through entries */

/*
	if you want to return a value as well
char *
nlget(nl, np)
NAMLIST nl;
int  * np;
*/

char   *
nlget(nl)
	NAMLIST *nl;
{
	STRLIST *p = nl->p;

	nl->p = nl->p->next;
	if (nl->p == 0)
		nl->p = nl->head;
/*	*np = p->val; */
	return (p->str);
}


/* return number of names in name list */
int
nlcount(nl)
	NAMLIST nl;
{
	return (nl.num);
}
void
nlclear(nl)
	NAMLIST *nl;
{
	STRLIST *tmp, *p = nl->p;

	while (p) {
		tmp = p->next;
		free(p);
		p = tmp;
	}
	nl->num = 0;
	nl->p = nl->head = nl->tail = 0;
}

void
nlinit(nl, max)
	NAMLIST *nl;
	int     max;
{
	nl->num = 0;
	nl->p = nl->head = nl->tail = 0;
	nl->max = max;
}

void
nlprint(nl)
	NAMLIST nl;
{
	int     i;
/*	int n; */
	int     count;
	char   *cp;
	NAMLIST *where;

	count = nlcount(nl);
	where = &nl;

	for (i = 0; i < count; i++) {
/*		cp = nlget(&where,&n);*/
/*		printf("name(%d) = >%s< val = %d\n", i, cp, n);*/
		cp = nlget(where);
		printf("name(%d) = >%s<\n", i, cp);
	}

	printf("\n");
}

int 
nldelete(nl, name)
	NAMLIST *nl;
	char   *name;
{
	STRLIST *namep;

	namep = strgetnode(name, nl->head, 1);
	if (namep != NULL) {
		strunlink(namep, &nl->head, &nl->tail);
		nl->p = nl->head;
		if (nl->num > 0) {
			nl->num--;
			if (nl->num == 0)
				nl->p = 0;	/* just in case */
		} else {
			mdb("tried to make nl.num negative");
		}
		return 0;
	} else {
		return -1;
	}
}

int 
nlpresent(name, nl)
	char    name[];
	NAMLIST *nl;
{
	if (nl->num > 0)
		return (strgetnode(name, nl->head, 1) != NULL);
	else
		return (NULL != NULL);
}

void
nldump(nl)
	NAMLIST nl;
{
	printf("head = %s\n", (char *) nl.head);
	printf("tail = %s\n", (char *) nl.tail);
	printf("p = %s\n", (char *) nl.p);
	printf("num = %d\n", nl.num);
	printf("max = %d\n", nl.max);
}
