/* Copyright 1991 by John Atwood deVries II. */
/* For copying and distribution information, see the file COPYING. */

/* primitive to handle group requests */

#include "server.h"
#include "externs.h"
#include "lookup.h"
#include "groups.h"
#include <assert.h>
#include <strings.h>

/* function declarations */
extern int find_group(char *);
extern int nldelete(NAMLIST *, char *);
extern void sendstatus(int, char *, char *);
extern int find_user(char *);
extern void senderror(int, char *);
extern void mdb(char *);
extern int nlpresent(char *, NAMLIST *);
extern void nlinit(NAMLIST *, int);
extern int find_empty_group();
extern void clear_group_item(int);
extern void nlput(NAMLIST *, char *);
extern char *getword(char *);
extern int nlcount(NAMLIST);
extern char *nlget(NAMLIST *);
extern int lookup(char *, char **);
extern char *get_tail(char *);
extern void sends_cmdout(int, char *);
extern void sendopen(int, int, char *);
void    s_status_group(int, int, int, char *, char *);
int     set_name(int, int, char *);
void    dump_table(int);


void
s_cancel(n, argc)
	int     n;
	int     argc;
{
	int     gi;
	int     is_restricted, is_moderator;
	int     can_do;
	int     dest;

	if (argc == 2) {
		/* constraints: group is restricted & inviter is mod &
		 * fields[1] is in invite list change in invite list otherwise
		 * complain appropriately side-effects: none other: fields[1]
		 * is nickname of person for cancel */
		/* gi is canceller's groupindex */
		gi = find_group(u_tab[n].group);

		is_restricted = (g_tab[gi].control == RESTRICTED);
		is_moderator = (g_tab[gi].mod == n);
		can_do = (is_restricted && is_moderator);

		if (can_do) {
			if (is_restricted && is_moderator) {
				if (nldelete(g_tab[gi].invites, fields[1]) == 0) {
					sprintf(mbuf, "%s cancelled",
					    fields[1]);
					sendstatus(n, "FYI", mbuf);
				} else {
					sprintf(mbuf, "%s was not invited",
					    fields[1]);
					sendstatus(n, "Warning", mbuf);
				}
			}
			if ((dest = find_user(fields[1])) < 0) {
				sprintf(mbuf,
				    "%s is not signed on", fields[1]);
				sendstatus(n, "Warning", mbuf);
			}
		} else {
			if (!is_restricted) {
				senderror(n, "The group isn't restricted.");
			}
			if (!is_moderator) {
				senderror(n, "You aren't the moderator.");
			}
		}
	} else {
		mdb("cancel: wrong number of parz");
	}
}
/* change group (if possible) */
/* group 1 is always:
	normal visibility
	public control
   otherwise the group is created:
	group name begins with .. => invisible
	group name begins with .  => secret
	otherwise visible
	n is mod
*/

int
s_change(n, argc)
	int     n;
	int     argc;
{
	int     group_exists;
	int     is_restricted;
	int     is_moderator;
	int     is_invited;
	int     isMyGroup;
	int     target_user;
	int     ngi, ogi;	/* new and old group indices */
	int     len, how_many;
	char    n_g_n[MAX_GROUPLEN + 1];	/* new group name */
	int	vis;

	if (argc == 2) {
		/* constraints: if group does not exist, there must be an
		 * empty slot in g_tab. if the group does exist, and is public
		 * or moderated none. if the group does exist, and is
		 * restricted n must have an invitation -or- n must be the
		 * mod. new group can't be the same as the old group. new
		 * group name has to be at least 1 char. new group name is
		 * limited in length.
		 * 
		 * notes: the old group doesn't exist if this is a login.
		 * presumes u_tab has an entry set up for n (except the group
		 * name and login members)
		 * 
		 * side-effects: if success, new group gets status "join" message
		 * if old group existed and still has members, old group gets
		 * status "leave" message otherwise old group gets destroyed
		 * if nec. return is 0 for either the old or new groups -- if
		 * QUIET, then no message.
		 * 
		 * if failure, message given and return is -1 */

		len = strlen(fields[1]);

		if (len <= 0) {
			/* oops, not long enough */
			senderror(n, "Null groupnames are not allowed.");
			return -1;
		}
		if (fields[1][0] != '.') {
			vis = VISIBLE;
			how_many = (MAX_GROUPLEN > len) ? len : MAX_GROUPLEN;
			bzero(n_g_n, MAX_GROUPLEN + 1);
			strncpy(n_g_n, fields[1], how_many);
		} else
			if (fields[1][1] == '.') {
				vis = INVISIBLE;
				len -= 2;
				how_many = (MAX_GROUPLEN > len) ? len : MAX_GROUPLEN;
				bzero(n_g_n, MAX_GROUPLEN + 1);
				strncpy(n_g_n, &fields[1][2], how_many);
			} else {
				vis = SECRET;
				len--;
				how_many = (MAX_GROUPLEN > len) ? len : MAX_GROUPLEN;
				bzero(n_g_n, MAX_GROUPLEN + 1);
				strncpy(n_g_n, &fields[1][1], how_many);
			}
		if (fields[1][0] == '@')
			if ((target_user = find_user(&fields[1][1])) < 0) {
				senderror(n, "User not found or is in a secret or invisible group.");
				return -1;
			} else
				strcpy(n_g_n, u_tab[target_user].group);

		group_exists = ((ngi = find_group(n_g_n)) >= 0);

		if (group_exists)
			is_invited = (nlpresent(u_tab[n].nickname,
				g_tab[ngi].invites));
		else
			is_invited = 0;

		if (fields[1][0] == '@') {
			isMyGroup = (find_group(u_tab[n].group) == ngi);
			if (((g_tab[ngi].visibility == SECRET) ||
				(g_tab[ngi].visibility == INVISIBLE)) &&
			    ((!is_invited) && (!isMyGroup))) {
				senderror(n, "User not found or is in a secret or invisible group.");
				return -1;
			}
		}
		if (!group_exists) {
			mdb("creating group");
			/* if it doesn't exist, try to create it */
			/* see if there is an empty slot */
			ngi = find_empty_group();
			if (ngi < 0) {
				/* complain and fail */
				senderror(n, "Too many groups!");
				return (-1);
			}
			/* could create it, so fill in the info */
			strcpy(g_tab[ngi].name, n_g_n);
			g_tab[ngi].visibility = vis;
			if (strcasecmp("1", n_g_n) != 0) {
				g_tab[ngi].control = MODERATED;
				g_tab[ngi].mod = n;
			} else {
				g_tab[ngi].control = PUBLIC;
			}
			/* set number of users in group to 1 */
			g_tab[ngi].num_users = 0;
			sprintf(mbuf, "%s, %d", g_tab[ngi].name,
				g_tab[ngi].num_users);
			mdb(mbuf);

			/* initialize the invitation list */
			nlinit(g_tab[ngi].invites, MAX_INVITES);
			/* nldump(*g_tab[ngi].invites); */
		}
		/* if we get here, the destination group exists */

		/* is it restricted? */

		is_restricted = (g_tab[ngi].control == RESTRICTED);
		if (is_restricted) {
			/* are we the moderator? */
			is_moderator = (g_tab[ngi].mod == n);
			if (!is_moderator) {
				/* are we invited? */
				is_invited = (nlpresent(
					u_tab[n].nickname,
					g_tab[ngi].invites));
				if (!is_invited) {
					sprintf(mbuf,
					    "%s is restricted.",
					    n_g_n);
					senderror(n, mbuf);
					sprintf(mbuf,
					    "%s tried to enter the group.",
					    u_tab[n].nickname);
					senderror(g_tab[ngi].mod, mbuf);
					return (-1);
				}
			}
		}
		/* remember if old group existed, and if so, what was its
		 * index */

		if (strlen(u_tab[n].group) == 0) {
			ogi = -1;
		} else {
			ogi = find_group(u_tab[n].group);
		}


		/* is the new group different than the old group? */
		if (ngi != ogi) {
			/* the group exists and we are allowed in. */
			strcpy(u_tab[n].group, n_g_n);
			g_tab[ngi].num_users++;
			sprintf(mbuf, "%s, %d users", g_tab[ngi].name,
				g_tab[ngi].num_users);
			mdb(mbuf);
			g_tab[ogi].num_users--;
			sprintf(mbuf, "%s, %d users", g_tab[ogi].name,
				g_tab[ogi].num_users);
			mdb(mbuf);

			/* tell the new group about the arrival */
			sprintf(mbuf, "%s (%s@%s) just arrived.",
			    u_tab[n].nickname,
			    u_tab[n].loginid,
			    u_tab[n].nodeid);
			if (g_tab[ngi].volume != QUIET) {
				if (u_tab[n].login == 0) {
					s_status_group(1, 0, n, "Signon", mbuf);
				} else {
					s_status_group(1, 0, n, "Join", mbuf);
				}
			}
			is_moderator = (g_tab[ngi].mod == n);
			if (!is_moderator) 
				sprintf(mbuf, "You are now in group %s",
				    u_tab[n].group);
			else
				sprintf(mbuf,
				"You are now in group %s as the moderator",
				u_tab[n].group);
				
			sendstatus(n, "Change", mbuf);

			/* did the old group exist? */
			if (ogi >= 0) {
				/* is there anyone left? */
				if (g_tab[ogi].num_users > 0) {
					/* tell them user left */
					sprintf(mbuf, "%s just left.",
					    u_tab[n].nickname);
					if (g_tab[ogi].volume != QUIET) {
						s_status_group(2, 0, ogi,
						    "Departure",
						    mbuf);
					}
				} else {
					/* zap the old group */
					clear_group_item(ogi);
				}
				return 0;
			}
		} else {
			senderror(n, "You are already in that group!");
			return -1;
		}
	} else {
		mdb("change: wrong number of parz");
		return (-1);
	}
	/* never reached */
	return 0;
}

void
s_invite(n, argc)
	int     n;
	int     argc;
{
	int     gi;
	int     is_restricted, is_moderator;
	int     can_do;
	int     dest;

	if (argc == 2) {
		/* constraints: group is restricted & inviter is mod =>
		 * messages and change in invite list group is not restricted
		 * => message only side-effects: if invitee is signed on, send
		 * status message other: fields[1] is nickname of person to
		 * invite */
/* mdb("finding group"); */
		gi = find_group(u_tab[n].group);	/* is inviter's group
							 * index */

/* mdb("checking restriction and stuff"); */
		is_restricted = (g_tab[gi].control == RESTRICTED);
		is_moderator = (g_tab[gi].mod == n);
		can_do = (is_restricted && is_moderator) || !is_restricted;

		if (can_do) {
/* mdb("supposedly can do"); */
			if (is_restricted && is_moderator) {
/* mdb("supposedly really will do"); */
				nlput(g_tab[gi].invites, fields[1]);
/* mdb("have put name on list"); */
				sprintf(mbuf, "%s invited", fields[1]);
				sendstatus(n, "FYI", mbuf);
/* mdb("have told inviter"); */
			}
			if ((dest = find_user(fields[1])) >= 0) {
				sprintf(mbuf,
				    "You are invited to group %s by %s",
				    u_tab[n].group,
				    u_tab[n].nickname);
				sendstatus(dest, "RSVP", mbuf);
			} else {
				sprintf(mbuf,
				    "%s is not signed on", fields[1]);
				sendstatus(n, "Warning", mbuf);
			}
		} else {
			if (!is_restricted) {
				senderror(n, "The group isn't restricted.");
			}
			if (!is_moderator) {
				senderror(n, "You aren't the moderator.");
			}
		}
	} else {
		mdb("invite: wrong number of parz");
	}
}

void
s_pass(n, argc)
	int     n;
	int     argc;
{
	int     is_moderator;
	int     gi;
	int     dest;
	char   *cp;

	if (argc == 2) {
		/* constraints: user must be mod of group they are in if
		 * passing, dest. nickname must be on
		 * 
		 * side-effects: if passed successfully, dest. nickname gets
		 * message if passed or relinquished successfully, user's
		 * moderated group gets message if the group is QUIET, no
		 * messages. */

		/* what is the user's group? */
		gi = find_group(u_tab[n].group);

		/* are they the mod? */
		is_moderator = (g_tab[gi].mod == n);

		if (!is_moderator) {
			senderror(n, "You are not the moderator!");
			return;
		}
		/* they are the mod */

		/* do they want to pass or relinquish? */
		if (strlen(fields[1]) == 0) {
			/* relinquish mod */
			g_tab[gi].mod = -1;
/*			g_tab[gi].control = PUBLIC; */

			/* tell people */
			sprintf(mbuf, "%s just relinquished the mod.",
			    u_tab[n].nickname);
			if (g_tab[gi].volume != QUIET) {
				s_status_group(1, 1, n, "Pass", mbuf);
			}
		} else {
			/* check dest. nickname */
			cp = getword(fields[1]);
			if (strlen(cp) == 0) {
				mdb("[Error] null string in pass");
				return;
			}
			/* get index of dest. nickname */
			dest = find_user(cp);

			/* do they exist? */
			if (dest >= 0) {
				/* fix the mod-ship */
				g_tab[gi].mod = dest;

				/* send message to dest. */
				sprintf(mbuf,
				    "%s passed the mod of %s to you.",
				    u_tab[n].nickname,
				    g_tab[gi].name);
				sendstatus(dest, "Pass", mbuf);

				/* send message to group */
				sprintf(mbuf,
				    "%s passed the mod to %s.",
				    u_tab[n].nickname, cp);
				if (g_tab[gi].volume != QUIET) {
					s_status_group(1, 1, n, "Pass", mbuf);
				}
			} else {
				sprintf(mbuf, "%s is not on!", cp);
				senderror(n, mbuf);
			}
		}
	} else {
		mdb("pass: wrong number of parz");
	}
}

void
s_boot(n, argc)
	int     n;
	int     argc;
{
	int     gi;
	int     is_moderator;
	int     dest;
	int     destgi;

	if (argc == 2) {
		/* constraints: inviter is mod & fields[1] is in this group
		 * put fields[1] in group 1 send boot message to fields[1]
		 * otherwise complain appropriately if group is QUIET, no
		 * messages sent. side-effects: if boot occurs, inform others
		 * in group other: fields[1] is nickname of person to boot */

		/* gi is canceller's groupindex */
		gi = find_group(u_tab[n].group);
		is_moderator = (g_tab[gi].mod == n);

		if (is_moderator) {
			dest = find_user(fields[1]);
			if (dest < 0) {
				sprintf(mbuf,
				    "%s is not signed on", fields[1]);
				sendstatus(n, "Warning", mbuf);
			} else {
				destgi = find_group(u_tab[dest].group);
				if (gi == destgi) {
					/* send status to group */
					sprintf(mbuf, "%s was booted.",
					    fields[1]);
					g_tab[destgi].num_users--;
					if (g_tab[destgi].volume != QUIET) {
						s_status_group(1, 1, n, "Boot", mbuf);
					}
					/* tell 'em it happened. */
					sprintf(mbuf, "%s booted you.",
					    u_tab[n].nickname);
					sendstatus(dest, "Boot", mbuf);
					/* fake an s_change to group 1 */
					strcpy(fields[1], "1");
					s_change(dest, 2);
/* note: although s_change returns a result, we presume that we can
         always join group 1. */
				} else {
					sprintf(mbuf,
					    "%s is not in your group",
					    fields[1]);
					senderror(n, mbuf);
				}
			}
		} else {
			if (!is_moderator) {
				senderror(n, "You aren't the moderator.");
			}
		}
	} else {
		mdb("boot: wrong number of parz");
	}
}

void
s_status(n, argc)
	int     n;
	int     argc;
{
	int     count;
	int     i;
	char   *cp;
	char   *vis;
	char   *con;
	char   *volume;
	char   *mod;
	int     is_moderator;
	int     is_public;
	int     is_group_1;
	int     can_do;
	char   *p1;
	char   *p2;
	int     gi;
	NAMLIST *ip;

	if (argc == 2) {
		if (strlen(fields[1]) == 0) {
			gi = find_group(u_tab[n].group);	/* is n's groupindex */

			/* visibility */
			switch (g_tab[gi].visibility) {

			case VISIBLE:
				vis = "Visible";
				break;
			case SECRET:
				vis = "Secret";
				break;
			case INVISIBLE:
				vis = "Invisible";
				break;
			default:
				mdb("s_status visibility bad");
				vis = "Unknown";
			}

			/* control */
			switch (g_tab[gi].control) {

			case PUBLIC:
				con = "Public";
				break;
			case MODERATED:
				con = "Moderated";
				break;
			case RESTRICTED:
				con = "Restricted";
				break;
			default:
				mdb("s_status control bad");
				con = "Unknown";
			}

			/* volume */
			switch (g_tab[gi].volume) {

			case QUIET:
				volume = "Quiet";
				break;
			case NORMAL:
				volume = "Normal";
				break;
			case LOUD:
				volume = "Loud";
				break;
			default:
				mdb("s_status volume bad");
				volume = "Unknown";
			}

			/* moderator */
			if (g_tab[gi].mod >= 0) {
				mod = u_tab[g_tab[gi].mod].nickname;
			} else {
				mod = "None";
			}

			sprintf(mbuf,
			    "%s Mod: %s (%s/ %s/ %s)",
			    u_tab[n].group, mod, vis, con, volume);
			sendstatus(n, "Name", mbuf);

			/* invitations */
			ip = g_tab[gi].invites;
			count = nlcount(*ip);
			for (i = 0; i < count; i++) {
				cp = nlget(ip);
				sendstatus(n, "Invited", cp);
			}

		} else {
			/* gi is n's group's index */
			gi = find_group(u_tab[n].group);

			is_public = (g_tab[gi].control == PUBLIC);
			is_moderator = (g_tab[gi].mod == n);
			is_group_1 = (strcasecmp(g_tab[gi].name,
				"1") == 0);
			can_do = ((is_moderator ||
				is_public ||
				(g_tab[gi].mod < 0)) && !is_group_1);

			if (!can_do) {
				if (is_group_1) {
					senderror(n,
					    "Can't change group 1!");
				} else {
					senderror(n,
					    "You aren't the moderator!");
				}
				return;
			}
			/* note: if the group is public when the command is
			 * given, then the whole command is executed even if
			 * the first option makes the group restricted. if
			 * there is no moderator and it isn't group 1, then
			 * the remaining users can still change some of the
			 * options, but still can't boot. also: this differs
			 * from Sean's server in that the moderator can make
			 * the group public while still retaining the mod and
			 * thus the ability to boot. but:  group 1 is still
			 * always public and visible. */
/*			p1=getword(fields[1]); */
			p1 = fields[1];
			while (strlen(p2 = getword(p1)) > 0) {
				switch (lookup(p2, status_table)) {
				case SET_RESTRICT:
					g_tab[gi].control =
					    RESTRICTED;
					cp = "restricted";
					for (i = 0; i < MAX_USERS; i++)
						if (strcasecmp(u_tab[i].group, g_tab[gi].name) == 0) {
							nlput(g_tab[gi].invites, u_tab[i].nickname);
							sprintf(mbuf, "%s invited", u_tab[i].nickname);
							sendstatus(g_tab[gi].mod, "FYI", mbuf);
							sprintf(mbuf, "You are invited to group %s by default.", g_tab[gi].name);
							sendstatus(i, "FYI", mbuf);
						}
					break;
				case SET_MODERATE:
					g_tab[gi].control =
					    MODERATED;
					nlinit(g_tab[gi].invites,
					    MAX_INVITES);
					cp = "moderated";
					break;
				case SET_PUBLIC:
					g_tab[gi].control =
					    PUBLIC;
					nlinit(g_tab[gi].invites,
					    MAX_INVITES);
					cp = "public";
					break;
				case SET_INVISIBLE:
					g_tab[gi].visibility =
					    INVISIBLE;
					cp = "invisible";
					break;
				case SET_SECRET:
					g_tab[gi].visibility =
					    SECRET;
					cp = "secret";
					break;
				case SET_VISIBLE:
					g_tab[gi].visibility =
					    VISIBLE;
					cp = "visible";
					break;
				case SET_QUIET:
					g_tab[gi].volume =
					    QUIET;
					cp = "quiet";
					break;
				case SET_NORMAL:
					g_tab[gi].volume =
					    NORMAL;
					cp = "normal";
					break;
				case SET_LOUD:
					g_tab[gi].volume =
					    LOUD;
					cp = "loud";
					break;
				case SET_NAME:
					p1 = get_tail(p1);
					p2 = getword(p1);
					if (set_name(n, gi, p2) < 0) {
						cp = "";
					} else {
						cp = "(changed name)";
					}
					break;
				case SET_QUESTION:
					dump_table(n);
					cp = NULL;
					break;
				default:
					sprintf(mbuf,
					    "Option %s unknown.",
					    p2);
					senderror(n, mbuf);
					cp = NULL;
				}
				if (cp != NULL) {
					sprintf(mbuf,
					    "Group status is now %s.",
					    cp);
					if (g_tab[gi].volume != QUIET) {
						s_status_group(1, 1, n, "Change", mbuf);
					}
				}
				p1 = get_tail(p1);
			}
		}
	} else {
		mdb("status: wrong number of parz");
	}
}

void
dump_table(n)
	int     n;
{
	sendstatus(n, "Info", "Table of status commands");
	sendstatus(n, "Info", "r - restricted m - moderated p - public");
	sendstatus(n, "Info", "i - invisible s - secret v - visible");
	sendstatus(n, "Info", "q - quiet n - normal l - loud");
	sendstatus(n, "Info", "name - change group name");
	sendstatus(n, "Info", "? -- this table");
}

void
s_topic(n, argc)
	int     n;
	int     argc;
{
	int     t_group;
	int     is_moderator;
	int     has_moderator;

	if (argc == 2) {
		t_group = find_group(u_tab[n].group);

		if (strlen(fields[1]) == 0) {
			if (strlen(g_tab[t_group].topic) == 0) {
				strcpy(mbuf, "(none)");
			} else {
				sprintf(mbuf, "The topic is: %s",
				    g_tab[t_group].topic);
			}
			sends_cmdout(n, mbuf);
			return;
		}
		is_moderator = (g_tab[t_group].mod == n);
		has_moderator = (g_tab[t_group].mod >= 0);

		if (has_moderator && (!is_moderator)) {
			senderror(n, "You are not the moderator.");
			return;
		}
		bzero(g_tab[n].topic, MAX_TOPICLEN + 1);
		strncpy(g_tab[t_group].topic, fields[1], MAX_TOPICLEN);
		if (g_tab[t_group].volume != QUIET) {
			sprintf(mbuf, "%s changed the topic to: \"%s\"",
			    u_tab[n].nickname,
			    g_tab[t_group].topic);
			if (g_tab[t_group].volume != QUIET) {
				s_status_group(1, 1, n, "Topic", mbuf);
			}
		}
	} else {
		mdb("topic: wrong number of parz");
	}
}
 /* send open group message */
 /* loop through the table... */
 /* make sure they aren't us unless echoback is on */
 /* look for other user being in this user's group */
 /* strings comparisons are done case-insensitive */
void
s_send_group(n)
	int     n;
{
	int     i;
	int     others = 0;
	char    my_group[MAX_GROUPLEN + 1];

	strcpy(my_group, u_tab[n].group);

	i = find_group(u_tab[n].group);
	if (g_tab[i].num_users < 2)
		senderror(n, "No one else in group!");
	else {
		/* send it to all of the real users */
		for (i = 0; i < MAX_REAL_USERS; i++) {
			if ((i != n) || u_tab[n].echoback) {
				if (strcasecmp(my_group, u_tab[i].group) == 0) {
					/* actually send it */
					others++;
					/* make sure they are still on */
					if (u_tab[i].login == 1) {
						sendopen(n, i, fields[0]);
					}
				}
			}
		}
	}
}
 /* send group status message */
 /* loop through the table... */
 /* look for other user being in this user's group */
 /* strings comparisons are done case-insensitive */
void
s_status_group(k, tellme, n, class_string, message_string)
	int     k;		/* 1 means n is u_tab index, otherwise n is
				 * g_tab index */
	int     tellme;		/* 0 means don't tell me (i.e. n), 1 means do
				 * tell me */
	int     n;
	char   *class_string;
	char   *message_string;
{
	int     i;
	char    my_group[MAX_GROUPLEN + 1];

	if (k == 1) {
		strcpy(my_group, u_tab[n].group);
		if (strlen(my_group) == 0)
			mdb("bad u_tab entry");
	} else {
		strcpy(my_group, g_tab[n].name);
		if (strlen(my_group) == 0)
			mdb("bad g_tab entry");
	}

	/* do it only for real users */
	for (i = 0; i < MAX_REAL_USERS; i++) {
		if (strcasecmp(my_group, u_tab[i].group) == 0) {
			/* n means something different here */
			/* for k=1 (u_tab entry) vs k=2 ( g_tab entry) */
			/* so n for k=2 doesn't make sense. */
			/* 'cuz it'd mean don't do this when user i was equal
			 * to group n... */
			if (k == 1) {
				if ((i != n) || tellme) {
					/* actually send it */
					sendstatus(i, class_string,
					    message_string);
				}
			} else {
				sendstatus(i, class_string, message_string);
			}
		}
	}
}

int
set_name(n, group, name)
	int     n;
	int     group;
	char   *name;
{
	char   *cp;
	int     i;
	int     len;
	int     how_many;
	char    n_g_n[MAX_GROUPLEN + 1];

	/* make sure they gave us a name to change to */
	if (strlen(name) == 0) {
		senderror(n, "New name may not be null.");
		return -1;
	}
	/* use only as much as needed */
	len = strlen(name);
	how_many = (MAX_GROUPLEN > len) ? len : MAX_GROUPLEN;
	bzero(n_g_n, MAX_GROUPLEN + 1);
	strncpy(n_g_n, name, how_many);

	if (!strcmp(n_g_n, "1")) {
		senderror(n, "Cannot change name to group 1.");
		return -1;
	}
	/* make sure that group doesn't already exist */
	if (find_group(n_g_n) >= 0) {
		senderror(n, "That group already exists.");
		return -1;
	}
	/* get a copy to the old name */
	cp = g_tab[group].name;

	/* check through the user table to see who is in that group */
	/* and change their group name entries */
	for (i = 0; i < MAX_REAL_USERS; i++) {
		if (strcasecmp(u_tab[i].group, cp) == 0) {
			strcpy(u_tab[i].group, n_g_n);
		}
	}

	/* finally change the name of the group itself */
	strcpy(g_tab[group].name, n_g_n);
	return 0;
}
