/* Copyright (c) 1991 by Carrick Sean Casey. */
/* For copying and distribution information, see the file COPYING. */

/* primitive to support the server logging function */

#include "server.h"
#include "config.h"
#include "externs.h"
#include <sys/time.h>
#include <sys/stat.h>

/* function prototypes */
extern void mdb(char *);
extern char *tildexpand(char *);
extern void linebuffer(FILE *);
extern void putl(char *, int);
int     logging();
void    closesessionlog();
int     startsessionlog(char *);


FILE   *logfp = NULL;

void
s_log(log_file)
	char   *log_file;
{

	if (!log_file) {
		if (logging())
			closesessionlog();
		else
			mdb("logging not on");
	} else {
		if (logging())
			mdb("logging already taking place");
		else
			startsessionlog(log_file);
	}
}


/* start a session log */
int
startsessionlog(path)
	char   *path;
{
	/* +++BUG+++ */
/*	int TRET; */

	char   *aptr, *ampm();
	time_t  time();
	struct tm *t, *localtime();
	time_t  clock;

	/* determine pathname to use */
	if (path == NULL || *path == '\0')
		if (gv.logfile)
			path = gv.logfile;
		else
			path = ICBDLOG;

	/* expand a tilde style path */
	if (*path == '~')
		if ((path = tildexpand(path)) == NULL) {
/*			strcpy(TRET, "s_log: bad login id in path"); */
			return (-1);
		}
	/* open the session log */
	if ((logfp = fopen(path, "a")) == NULL) {
/*		if (errno > sys_nerr)
		   sprintf(TRET,
		    "s_log: can't open \"%s\": errno %d", path, errno);
		else
		   sprintf(TRET,
		    "s_log: can't open \"%s\": %s", path, sys_errlist[errno]);
*/
		return (-1);
	}
	/* protect the logfile against others */
	if (fchmod((int) (fileno(logfp)), 0600) != 0) {
/*		if (errno > sys_nerr)
		   sprintf(TRET,
		    "s_log: can't fchmod \"%s\": errno %d", path, errno);
		else
		   sprintf(TRET,
		    "s_log: can't fchmod \"%s\": %s", path, sys_errlist[errno]);
*/
		fclose(logfp);
		logfp = NULL;
		return (-1);
	}
	/* line buffer it */
	linebuffer(logfp);

	/* get the time so we can timestamp it */
	time(&clock);
	t = localtime(&clock);
	aptr = ampm(clock, 0);
	if (*aptr == ' ')
		aptr++;

	/* timestamp it */
	sprintf(mbuf, "Session log \"%s\" started at %d/%02d/%-2d %s.",
	    path, t->tm_mon + 1, t->tm_mday, t->tm_year, aptr);
	putl(mbuf, PL_SCR | PL_LOG);

	return (0);
}


/* close the logfile */
void
closesessionlog()
{
	char   *aptr, *ampm();
	time_t  time();
	struct tm *t, *localtime();
	time_t  clock;

	/* do nothing if we're not really logging */
	if (logfp == NULL)
		return;

	/* get the time so we can timestamp it (deja vu?) */
	time(&clock);
	t = localtime(&clock);
	aptr = ampm(clock, 0);
	if (*aptr == ' ')
		aptr++;

	/* timestamp it */
	sprintf(mbuf, "Session log closed at %d/%02d/%-2d %s.",
	    t->tm_mon + 1, t->tm_mday, t->tm_year, aptr);
	putl(mbuf, PL_SCR | PL_LOG);

	/* close it */
	fclose(logfp);
	logfp = NULL;
	return;
}


/* return 1 if session logging is in effect, otherwise return 0 */
int
logging()
{
	if (logfp == NULL)
		return (0);
	return (1);
}
