/* Copyright (c) 1991 by John Atwood deVries II. */
/* For copying and distribution information, see the file COPYING. */

/* send various messages to the client */

#include "server.h"
#include "externs.h"
#include "../h/protocol.h"
#include "../h/version.h"
#include <strings.h>

/* function prototypes */
extern void autoCommand(int);
extern int _sendpacket(int, char *);
void    doSend(int, int);
void    autoBeep(int);


/* send normal open group message to the client */
void
sendopen(n, i, txt)
	int     n;
	int     i;
	char   *txt;
{
	/* construct open message */
	sprintf(pbuf, "%c%s\001%s", M_OPEN, u_tab[n].nickname, txt);
	doSend(n, i);
}
/* send an exit message to the client -- makes the client disconnect */
void
sendexit(n)
	int     n;
{
	/* construct exit message */
	sprintf(pbuf, "%c", M_EXIT);
	doSend(-1, n);
}
/* send a ping */
void
sendping(n)
	int     n;
{
	/* construct pint message */
	sprintf(pbuf, "%c", M_PING);
	doSend(-1, n);
}


/* send an error message to the client */
void
senderror(n, error_string)
	int     n;
	char   *error_string;
{
	sprintf(pbuf, "%c%s", M_ERROR, error_string);
	doSend(-1, n);
}
/* send a status message to the client */
void
sendstatus(n, class_string, message_string)
	int     n;
	char   *class_string;
	char   *message_string;
{
	sprintf(pbuf, "%c%s\001%s", M_STATUS, class_string, message_string);
	doSend(-1, n);
}
/* send simple command output message to the client */
void
sends_cmdout(n, output_string)
	int     n;
	char   *output_string;
{
	sprintf(pbuf, "%c%s\001%s", M_CMDOUT, "co", output_string);
	doSend(-1, n);
}
/* send personal message to a client */
void
sendperson(from, to, message_string)
	int     from;
	int     to;
	char   *message_string;
{
	sprintf(pbuf, "%c%s\001%s", M_PERSONAL, u_tab[from].nickname,
	    message_string);
	doSend(from, to);
}
/* send beep message to a client */
void
sendbeep(from, to)
	int     from;
	int     to;
{
	sprintf(pbuf, "%c%s", M_BEEP, u_tab[from].nickname);
	doSend(from, to);
}

void
s_new_user(n)
	int     n;		/* fd of their socket */
{
	/* construct proto(col) message */
	sprintf(pbuf, "%c%d\001%s\001%s", M_PROTO, PROTO_LEVEL,
	    thishost, VERSION);
	doSend(-1, n);
}

void
send_loginok(n)
	int     n;
{
	/* construct loginok message */
	sprintf(pbuf, "%c", M_LOGINOK);
	doSend(-1, n);
}
/* send an important message to the client */
void
sendimport(n, class_string, output_string)
	int     n;
	char   *class_string;
	char   *output_string;
{
	sprintf(pbuf, "%c%s\001%s", M_IMPORTANT, class_string,
	    output_string);
	doSend(-1, n);
}


void
send_info(n, loginid, nodeid)
	int     n;
	char   *loginid;
	char   *nodeid;
{
	sprintf(mbuf, "%s@%s", loginid, nodeid);
	sprintf(pbuf, "%c%s\001%s", M_CMDOUT, "in", mbuf);
	doSend(-1, n);
}

void
doSend(from, to)
	int     from;
	int     to;
{
	if (to >= MAX_REAL_USERS) {
		*pp = ' ';
		switch (*(pp + 1)) {
		case M_BEEP:	/* someone beeped us */
			autoBeep(from);
			break;
		case M_PERSONAL:	/* someone sent us a message */
			autoCommand(from);
			break;
		default:	/* do nothing -- ignore */
			break;
		}
	} else {
		int     ret;

		setlen;
		ret = _sendpacket(to, pp);
		if (debug && ret < 0)
			perror("doSend");
	}
}

void
autoBeep(n)
	int     n;
{
	sendperson(NICKSERV, n, "Beep yerself!");
	u_tab[NICKSERV].t_recv = time(NULL);
}
/*
	Copyright (c) 1991 by Keith Graham
	Modifications Copyright (c) 1991 by John Atwood deVries II

	"to" is the destination socket..
*/

void
user_wline(to, mod, nick, idle, resp, login, user, site, name)
	int     to;
	char   *mod, *nick;
	int     idle, resp, login;
	char   *user, *site, *name;
{
	sprintf(pbuf, "%cwl\001%s\001%s\001%d\001%d\001%d\001%s\001%s\001%s%c",
	    M_CMDOUT, mod, nick, idle, resp, login, user, site, name, '\0');
	doSend(-1, to);
}

void
user_wgroupline(to, group, topic, c, d)
	int     to;
	char   *group;
	char   *topic;
	int     c, d;
{
	sprintf(pbuf, "%cwg\001%s\001%s\001%d\001%d%c",
	    M_CMDOUT, group, topic, c, d, '\0');
	doSend(-1, to);
}

void
user_ghead(to)
	int     to;
{
	sprintf(pbuf, "%cgh%c", M_CMDOUT, '\0');
	doSend(-1, to);
}

void
user_whead(to)
	int     to;
{
	sprintf(pbuf, "%cwh%c", M_CMDOUT, '\0');
	doSend(-1, to);
}
