/* Copyright (c) 1991 by John Atwood deVries II. */
/* For copying and distribution information, see the file COPYING. */

/* user table manipulation */

#include "server.h"
#include "groups.h"
#include "externs.h"
#include <strings.h>

/* clear a particular user entry */
void
clear_user_item(n)
	int     n;
{
	bzero(u_tab[n].loginid, MAX_IDLEN + 1);
	bzero(u_tab[n].nodeid, MAX_NODELEN + 1);
	bzero(u_tab[n].nickname, MAX_NICKLEN + 1);
	bzero(u_tab[n].password, MAX_PASSWDLEN + 1);
	bzero(u_tab[n].realname, MAX_REALLEN + 1);
	bzero(u_tab[n].group, MAX_GROUPLEN + 1);
	u_tab[n].login = 0;
	u_tab[n].echoback = 0;
	u_tab[n].flags = 0;
	u_tab[n].t_on = (time_t) 0;
	u_tab[n].t_sent = (time_t) 0;
	u_tab[n].t_recv = (time_t) 0;
}
/* fill a particular user entry */
void
fill_user_entry(n, loginid, nodeid, nickname, password, group, mylogin,
    echoback)
	int     n;
	char   *loginid;
	char   *nodeid;
	char   *nickname;
	char   *password;
	char   *group;
	int     mylogin;
	int     echoback;
{
	int     len;
	int     how_many;

	strcpy(u_tab[n].loginid, loginid);
	strcpy(u_tab[n].nodeid, nodeid);

	len = strlen(nickname);
	how_many = (len > MAX_NICKLEN) ? MAX_NICKLEN : len;
	strncpy(u_tab[n].nickname, nickname, how_many);

	len = strlen(password);
	how_many = (len > MAX_PASSWDLEN) ? MAX_PASSWDLEN : len;
	strncpy(u_tab[n].password, password, how_many);

	strcpy(u_tab[n].group, group);
	u_tab[n].login = mylogin;
	u_tab[n].echoback = echoback;
}


/* clear the entire user table */
void
clear_users()
{
	int     i;

	for (i = 0; i < MAX_USERS; i++) {
		clear_user_item(i);
	}
}

/* find a slot in the user table with a particular name */
/* return that index if found, -1 otherwise */
/* case insensitive */

int
find_user(name)
	char   *name;
{
	int     result = -1;
	int     i;

	/* don't attempt to find if null */
	if (strlen(name) == 0)
		return result;

	/* search through table */
	for (i = 0; i < MAX_USERS; i++) {
		if (strcasecmp(u_tab[i].nickname, name) == 0) {
			result = i;
			break;
		}
	}
	return result;
}

void
dump_user(n)
	int     n;
{
	printf("loginid: %s\n", u_tab[n].loginid);
	printf("nodeid: %s\n", u_tab[n].nodeid);
	printf("nickname: %s\n", u_tab[n].nickname);
	printf("password: %s\n", u_tab[n].password);
	printf("realname: %s\n", u_tab[n].realname);
	printf("group: %s\n", u_tab[n].group);
	printf("login: %d\n", u_tab[n].login);
	printf("echoback: %d\n", u_tab[n].echoback);
}
